@@ -1,28 +1,41 @@
-#!/usr/bin/env python
 from __future__ import print_function
+import brotli
+import io
 import socket
 import ssl
-import string
 import errno
 import time
-import StringIO
 import gzip
 import os
 import sys
 import re
 import base64
 import zlib
-import Cookie
 import encodings
 from IPy import IP
-import errors
 
-reload(sys)
-sys.setdefaultencoding('utf8')
+from six import BytesIO, PY2, ensure_binary, ensure_str, iteritems, \
+    text_type
+from six.moves import http_cookies
+
+from . import errors
+
+
+# Fallback to PROTOCOL_SSLv23 if PROTOCOL_TLS is not available.
+PROTOCOL_TLS = getattr(ssl, "PROTOCOL_TLS", ssl.PROTOCOL_SSLv23)
+
+
+if PY2:
+    reload(sys)  # pragma: no flakes
+    sys.setdefaultencoding('utf8')
+    escape_codec = 'string_escape'
+else:
+    escape_codec = 'unicode_escape'
+
 
 class HttpResponse(object):
     def __init__(self, http_response, user_agent):
-        self.response = http_response
+        self.response = ensure_binary(http_response)
         # For testing purposes HTTPResponse might be called OOL
         try:
             self.dest_addr = user_agent.request_object.dest_addr
@@ -35,7 +48,7 @@ class HttpResponse(object):
         self.version = None
         self.headers = None
         self.data = None
-        self.CRLF = '\r\n'
+        self.CRLF = b'\r\n'
         self.process_response()
 
     def parse_content_encoding(self, response_headers, response_data):
@@ -43,13 +56,38 @@ class HttpResponse(object):
         Parses a response that contains Content-Encoding to retrieve
         response_data
         """
-        if response_headers['content-encoding'] == 'gzip':
-            buf = StringIO.StringIO(response_data)
+        if response_headers['content-encoding'] == 'br':
+            try:
+                response_data = brotli.decompress(response_data)
+            except brotli.error:
+                raise errors.TestError(
+                    'Invalid or missing brotli data found',
+                    {
+                        'response_data': str(response_data),
+                        'function': 'http.HttpResponse.parse_content_encoding'
+                    })
+        elif response_headers['content-encoding'] == 'gzip':
+            buf = BytesIO(response_data)
             zipbuf = gzip.GzipFile(fileobj=buf)
-            response_data = zipbuf.read()
+            try:
+                response_data = zipbuf.read()
+            except IOError:
+                raise errors.TestError(
+                    'Invalid or missing gzip data found',
+                    {
+                        'response_data': str(response_data),
+                        'function': 'http.HttpResponse.parse_content_encoding'
+                    })
         elif response_headers['content-encoding'] == 'deflate':
-            data = StringIO.StringIO(zlib.decompress(response_data))
-            response_data = data.read()
+            try:
+                response_data = zlib.decompress(response_data, -zlib.MAX_WBITS)
+            except zlib.error:
+                raise errors.TestError(
+                    'Invalid or missing deflate data found',
+                    {
+                        'response_data': str(response_data),
+                        'function': 'http.HttpResponse.parse_content_encoding'
+                    })
         else:
             raise errors.TestError(
                 'Received unknown Content-Encoding',
@@ -69,7 +107,8 @@ class HttpResponse(object):
         except ValueError:
             origin_is_ip = False
         for cookie_morsals in cookie.values():
-            # If the coverdomain is blank or the domain is an IP set the domain to be the origin
+            # If the coverdomain is blank or the domain is an IP
+            # set the domain to be the origin
             if cookie_morsals['domain'] == '' or origin_is_ip:
                 # We want to always add a domain so it's easy to parse later
                 return (cookie, self.dest_addr)
@@ -85,9 +124,10 @@ class HttpResponse(object):
                         first_non_dot = i
                         break
                 cover_domain = cover_domain[first_non_dot:]
-                # We must parse the coverDomain to make sure its not in the suffix list
+                # We must parse the coverDomain to make sure its
+                # not in the suffix list
                 psl_path = os.path.dirname(__file__) + os.path.sep + \
-                'util' + os.path.sep + 'public_suffix_list.dat'
+                    'util' + os.path.sep + 'public_suffix_list.dat'
                 # Check if the public suffix list is present in the ftw dir
                 if os.path.exists(psl_path):
                     pass
@@ -99,9 +139,10 @@ class HttpResponse(object):
                             'function': 'http.HttpResponse.check_for_cookie'
                         })
                 try:
-                    with open(psl_path, 'r') as public_suffixs:
-                        for line in public_suffixs:
-                            if line[:2] == '//' or line[0] == ' ' or line[0].strip() == '':
+                    with io.open(psl_path, 'r', encoding='utf-8') as fo:
+                        for line in fo:
+                            if line[:2] == '//' or line[0] == ' ' or \
+                               line[0].strip() == '':
                                 continue
                             if cover_domain == line.strip():
                                 return False
@@ -114,7 +155,7 @@ class HttpResponse(object):
                         })
                 # Generate Origin Domain TLD
                 i = self.dest_addr.rfind('.')
-                o_tld = self.dest_addr[i+1:]
+                o_tld = self.dest_addr[i + 1:]
                 # if our cover domain is the origin TLD we ignore
                 # Quick sanity check
                 if cover_domain == o_tld:
@@ -144,18 +185,18 @@ class HttpResponse(object):
         Parses an HTTP response after an HTTP request is sent
         """
         split_response = self.response.split(self.CRLF)
-        response_line = split_response[0]
+        response_line = ensure_str(split_response[0])
         response_headers = {}
         response_data = None
         data_line = None
         for line_num in range(1, len(split_response[1:])):
             # CRLF represents the start of data
-            if split_response[line_num] == '':
+            if not split_response[line_num]:
                 data_line = line_num + 1
                 break
             else:
                 # Headers are all split by ':'
-                header = split_response[line_num].split(':', 1)
+                header = split_response[line_num].split(b':', 1)
                 if len(header) != 2:
                     raise errors.TestError(
                         'Did not receive a response with valid headers',
@@ -163,12 +204,13 @@ class HttpResponse(object):
                             'header_rcvd': str(header),
                             'function': 'http.HttpResponse.process_response'
                         })
+                header = ensure_str(header[0]), ensure_str(header[1])
                 response_headers[header[0].lower()] = header[1].lstrip()
         if 'set-cookie' in response_headers.keys():
             try:
-                cookie = Cookie.SimpleCookie()
+                cookie = http_cookies.SimpleCookie()
                 cookie.load(response_headers['set-cookie'])
-            except Cookie.CookieError as err:
+            except http_cookies.CookieError as err:
                 raise errors.TestError(
                     'Error processing the cookie content into a SimpleCookie',
                     {
@@ -217,6 +259,7 @@ class HttpResponse(object):
         self.headers = response_headers
         self.data = response_data
 
+
 class HttpUA(object):
     """
     Act as the User Agent for our regression testing
@@ -253,9 +296,8 @@ class HttpUA(object):
                 {
                     'msg': err,
                     'function': 'http.HttpUA.send_request'
-                })  			
-        finally:
-            self.get_response()
+                })
+        self.get_response()
 
     def build_socket(self):
         """
@@ -267,7 +309,10 @@ class HttpUA(object):
             self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
             # Check if TLS
             if self.request_object.protocol == 'https':
-                self.sock = ssl.wrap_socket(self.sock, ciphers=self.CIPHERS)
+                context = ssl.SSLContext(PROTOCOL_TLS)
+                context.set_ciphers(self.CIPHERS)
+                self.sock = context.wrap_socket(
+                    self.sock, server_hostname=self.request_object.dest_addr)
             self.sock.connect(
                 (self.request_object.dest_addr, self.request_object.port))
         except socket.error as msg:
@@ -305,13 +350,13 @@ class HttpUA(object):
     def build_request(self):
         request = '#method# #uri##version#%s#headers#%s#data#' % \
                   (self.CRLF, self.CRLF)
-        request = string.replace(
-            request, '#method#', self.request_object.method)
+        request = request.replace(
+            '#method#', self.request_object.method)
         # We add a space after here to account for HEAD requests with no url
-        request = string.replace(
-            request, '#uri#', self.request_object.uri + ' ')
-        request = string.replace(
-            request, '#version#', self.request_object.version)
+        request = request.replace(
+            '#uri#', self.request_object.uri + ' ')
+        request = request.replace(
+            '#version#', self.request_object.version)
         available_cookies = self.find_cookie()
         # If the user has requested a tracked cookie and we have one set it
         if available_cookies:
@@ -319,35 +364,43 @@ class HttpUA(object):
             if 'cookie' in self.request_object.headers.keys():
                 # Create a SimpleCookie out of our provided cookie
                 try:
-                    provided_cookie = Cookie.SimpleCookie()
+                    provided_cookie = http_cookies.SimpleCookie()
                     provided_cookie.load(self.request_object.headers['cookie'])
-                except Cookie.CookieError as err:
+                except http_cookies.CookieError as err:
                     raise errors.TestError(
-                        'Error processing the existing cookie into a SimpleCookie',
+                        'Error processing the existing cookie into a '
+                        'SimpleCookie',
                         {
                             'msg': str(err),
-                            'set_cookie': str(self.request_object.headers['cookie']),
+                            'set_cookie':
+                                str(self.request_object.headers['cookie']),
                             'function': 'http.HttpResponse.build_request'
                         })
                 result_cookie = {}
-                for cookie_key, cookie_morsal in provided_cookie.iteritems():
-                    result_cookie[cookie_key] = provided_cookie[cookie_key].value
+                for cookie_key, cookie_morsal in iteritems(provided_cookie):
+                    result_cookie[cookie_key] = \
+                        provided_cookie[cookie_key].value
                 for cookie in available_cookies:
-                    for cookie_key, cookie_morsal in cookie.iteritems():
+                    for cookie_key, cookie_morsal in iteritems(cookie):
                         if cookie_key in result_cookie.keys():
-                            # we don't overwrite a user specified cookie with a saved one
+                            # we don't overwrite a user specified
+                            # cookie with a saved one
                             pass
                         else:
-                            result_cookie[cookie_key] = cookie[cookie_key].value
-                for key, value in result_cookie.iteritems():
-                    cookie_value += (unicode(key) + '=' + unicode(value) + '; ')
+                            result_cookie[cookie_key] = \
+                                cookie[cookie_key].value
+                for key, value in iteritems(result_cookie):
+                    cookie_value += (text_type(key) + '=' +
+                                     text_type(value) + '; ')
                     # Remove the trailing semicolon
                 cookie_value = cookie_value[:-2]
                 self.request_object.headers['cookie'] = cookie_value
             else:
                 for cookie in available_cookies:
-                    for cookie_key, cookie_morsal in cookie.iteritems():
-                        cookie_value += (unicode(cookie_key) + '=' + unicode(cookie_morsal.coded_value) + '; ')
+                    for cookie_key, cookie_morsal in iteritems(cookie):
+                        cookie_value += (text_type(cookie_key) + '=' +
+                                         text_type(cookie_morsal.coded_value) +
+                                         '; ')
                         # Remove the trailing semicolon
                     cookie_value = cookie_value[:-2]
                     self.request_object.headers['cookie'] = cookie_value
@@ -355,23 +408,29 @@ class HttpUA(object):
         # Expand out our headers into a string
         headers = ''
         if self.request_object.headers != {}:
-            for hname, hvalue in self.request_object.headers.iteritems():
-                headers += unicode(hname) + ': ' + unicode(hvalue) + self.CRLF
-        request = string.replace(request, '#headers#', headers)
+            for hname, hvalue in iteritems(self.request_object.headers):
+                headers += text_type(hname) + ': ' + \
+                    text_type(hvalue) + self.CRLF
+        request = request.replace('#headers#', headers)
 
         # If we have data append it
         if self.request_object.data != '':
             # Before we do that see if that is a charset
             encoding = "utf-8"
-            # Check to see if we have a content type and magic is off (otherwise UTF-8)
-            if 'Content-Type' in self.request_object.headers.keys() and self.request_object.stop_magic is False:
+            # Check to see if we have a content type and magic is
+            # off (otherwise UTF-8)
+            if 'Content-Type' in self.request_object.headers.keys() and \
+               self.request_object.stop_magic is False:
                 pattern = re.compile(r'\;\s{0,1}?charset\=(.*?)(?:$|\;|\s)')
-                m = re.search(pattern, self.request_object.headers['Content-Type'])
+                m = re.search(pattern,
+                              self.request_object.headers['Content-Type'])
                 if m:
-                    possible_choices = list(set(encodings.aliases.aliases.keys())) + list(set(encodings.aliases.aliases.values()))
+                    possible_choices = \
+                        list(set(encodings.aliases.aliases.keys())) + \
+                        list(set(encodings.aliases.aliases.values()))
                     choice = m.group(1)
                     # Python will allow these aliases but doesn't list them
-                    choice = choice.replace('-','_')
+                    choice = choice.replace('-', '_')
                     choice = choice.lower()
                     if choice in possible_choices:
                         encoding = choice
@@ -382,13 +441,14 @@ class HttpUA(object):
                     'Error encoding the data with the charset specified',
                     {
                         'msg': str(err),
-                        'Content-Type': str(self.request_object.headers['Content-Type']),
-                        'data': unicode(self.request_object.data),
+                        'Content-Type':
+                            str(self.request_object.headers['Content-Type']),
+                        'data': text_type(self.request_object.data),
                         'function': 'http.HttpResponse.build_request'
-                    })                
-            request = string.replace(request, '#data#', data)
+                    })
+            request = request.replace('#data#', ensure_str(data))
         else:
-            request = string.replace(request, '#data#', '')
+            request = request.replace('#data#', '')
         # If we have a Raw Request we should use that instead
         if self.request_object.raw_request is not None:
             if self.request_object.encoded_request is not None:
@@ -397,15 +457,15 @@ class HttpUA(object):
                     {
                         'function': 'http.HttpUA.build_request'
                     })
-            request = self.request_object.raw_request
-            # We do this regardless of magic if you want to send a literal 
+            request = ensure_binary(self.request_object.raw_request)
+            # We do this regardless of magic if you want to send a literal
             # '\' 'r' or 'n' use encoded request.
-            request = request.decode('string_escape')
+            request = request.decode(escape_codec)
         if self.request_object.encoded_request is not None:
             request = base64.b64decode(self.request_object.encoded_request)
-            request = request.decode('string_escape')
+            request = request.decode(escape_codec)
         # if we have an Encoded request we should use that
-        self.request = request
+        self.request = ensure_binary(request)
 
     def get_response(self):
         """
@@ -426,7 +486,7 @@ class HttpUA(object):
             try:
                 data = self.sock.recv(self.RECEIVE_BYTES)
                 if data:
-                    our_data.append(data)
+                    our_data.append(ensure_binary(data))
                     begin = time.time()
                 else:
                     # Sleep for sometime to indicate a gap
@@ -437,11 +497,11 @@ class HttpUA(object):
                     pass
                 # SSL will return SSLWantRead instead of EAGAIN
                 elif sys.platform == 'win32' and \
-                err.errno == errno.WSAEWOULDBLOCK:
-                        pass
+                        err.errno == errno.WSAEWOULDBLOCK:
+                    pass
                 elif (self.request_object.protocol == 'https' and
-                    err[0] == ssl.SSL_ERROR_WANT_READ):
-                        continue
+                      err.args[0] == ssl.SSL_ERROR_WANT_READ):
+                    continue
                 # If we didn't it's an error
                 else:
                     raise errors.TestError(
@@ -453,19 +513,8 @@ class HttpUA(object):
                             'message': err,
                             'function': 'http.HttpUA.get_response'
                         })
-        if ''.join(our_data) == '':
-            raise errors.TestError(
-                'No response from server. Request likely timed out.',
-                {
-                    'host': self.request_object.dest_addr,
-                    'port': self.request_object.port,
-                    'proto': self.request_object.protocol,
-                    'msg': 'Please send the request and check Wireshark',
-                    'function': 'http.HttpUA.get_response'
-                })                                    
-        self.response_object = HttpResponse(''.join(our_data), self)
         try:
-            self.sock.shutdown(1)
+            self.sock.shutdown(socket.SHUT_WR)
             self.sock.close()
         except socket.error as err:
             raise errors.TestError(
@@ -473,4 +522,17 @@ class HttpUA(object):
                 {
                     'msg': err,
                     'function': 'http.HttpUA.get_response'
-                })  
+                })
+        else:
+            self.response_object = HttpResponse(b''.join(our_data), self)
+        finally:
+            if not b''.join(our_data):
+                raise errors.TestError(
+                    'No response from server. Request likely timed out.',
+                    {
+                        'host': self.request_object.dest_addr,
+                        'port': self.request_object.port,
+                        'proto': self.request_object.protocol,
+                        'msg': 'Please send the request and check Wireshark',
+                        'function': 'http.HttpUA.get_response'
+                    })
