@@ -1,14 +1,14 @@
 import abc
+import six
 
 
+@six.add_metaclass(abc.ABCMeta)
 class LogChecker():
     """
     LogChecker is an abstract class that integrations with WAFs MUST implement.
     This class is used by the testrunner to test log lines against an expected
     regex
     """
-    __metaclass__ = abc.ABCMeta
-
     def __init__(self):
         self.start = None
         self.end = None
