@@ -1,6 +1,7 @@
 import pytest
 from ftw import testrunner, errors
 
+
 @pytest.fixture
 def modsec_logger_obj():
     """
@@ -9,11 +10,12 @@ def modsec_logger_obj():
     """
     return None
 
+
 def test_modsecurityv2(modsec_logger_obj, ruleset, test, destaddr):
     """
     Modsec specific test
     """
-    runner = testrunner.TestRunner() 
+    runner = testrunner.TestRunner()
     try:
         for stage in test.stages:
             if destaddr is not None:
@@ -21,5 +23,5 @@ def test_modsecurityv2(modsec_logger_obj, ruleset, test, destaddr):
             runner.run_stage(stage, modsec_logger_obj)
     except errors.TestError as e:
         e.args[1]['meta'] = ruleset.meta
-        pytest.fail('Failure! Message -> {0}, Context -> {1}'
-                        .format(e.args[0],e.args[1]))
+        pytest.fail('Failure! Message -> {0}, Context -> {1}'.format(
+                    e.args[0], e.args[1]))
